@@ -106,14 +106,14 @@ t_cached_object *add_to_cache(t_session *session, char *file) {
 	int fd, i;
 	size_t bytes_read, bytes_total = 0;
 
-	if ((size = filesize(file)) == -1) {
+	if (stat(file, &status) == -1) {
+		return NULL;
+	} else if ((size = status.st_size) == -1) {
 		return NULL;
 	} else if ((size < session->config->cache_min_filesize) || (size > session->config->cache_max_filesize)) {
 		return NULL;
 	} else if (cachesize + size > session->config->cache_size) {
 		return NULL;
-	} else if (stat(file, &status) == -1) {
-		return NULL;
 	} else if ((object = (t_cached_object*)malloc(sizeof(t_cached_object))) == NULL) {
 		return NULL;
 	} else if ((object->file = strdup(file)) == NULL) {
