@@ -4,8 +4,8 @@
 #include <stdbool.h>
 #include "global.h"
 
-#define TIME_IN_CACHE     60
-#define MAX_CACHE_TIMER 3600
+#define TIME_IN_CACHE     MINUTE
+#define MAX_CACHE_TIMER     HOUR
 
 typedef struct type_cached_object {
 	char          *file;
