@@ -16,9 +16,8 @@
 #include "libstr.h"
 #include "libfs.h"
 #include "userconfig.h"
-#ifndef HAVE_SETENV
-#include "envir.h"
-#endif
+
+#define DELIMITER ':'
 
 typedef struct type_wrap {
 	char *cgiroot;
@@ -31,7 +30,10 @@ typedef struct type_wrap {
 int cgi_pid;
 
 void ALRM_handler() {
-	kill(cgi_pid, 9);
+	if (kill(cgi_pid, 15) != -1) {
+		sleep(1);
+		kill(cgi_pid, 9);
+	}
 }
 
 void print_code(int code) {
@@ -125,7 +127,7 @@ bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data, ch
 
 					/* Wrap Id
 					 */
-					if (split_string(rest, &item, &rest, ':') == -1) {
+					if (split_string(rest, &item, &rest, DELIMITER) == -1) {
 						break;
 					}
 					if (strcmp(item, wrapid) != 0) {
@@ -134,7 +136,7 @@ bool get_wrap_data(char *wrapid, char *handler, char *cgi, t_wrap *wrap_data, ch
 
 					/* Homedirectory
 					 */
-					if (split_string(rest, &item, &rest, ':') == -1) {
+					if (split_string(rest, &item, &rest, DELIMITER) == -1) {
 						break;
 					}
 					if (*item == '/') {
