@@ -17,13 +17,15 @@
 #include <netinet/in.h>
 #endif
 #include "global.h"
+#include "alternative.h"
 #include "libstr.h"
 #include "send.h"
 #include "cgi.h"
+#include "serverconfig.h"
 #include "envir.h"
 #include "log.h"
 
-#define MAX_CGI_DELAY_TIMER 60
+#define MAX_CGI_DELAY_TIMER MINUTE
 #define DUMMY_BUFFER_SIZE 65536
 
 static int delay_timer = 0;
@@ -242,6 +244,12 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 			fcntl(STDOUT_FILENO, F_SETFD, 0);
 			fcntl(STDERR_FILENO, F_SETFD, 0);
 
+			/* Close all other open filedescriptors.
+			 */
+			close(session->client_socket);
+			close_bindings(session->config->binding);
+			log_close(session->config->first_host, true);
+
 			set_environment(session, NULL);
 
 			if ((slash = strrchr(session->file_on_disk, '/')) != NULL) {
@@ -307,7 +315,12 @@ pid_t fork_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 			/* Send POST data to CGI program.
 			 */
 			if ((session->body != NULL) && (session->content_length > 0)) {
-				write(cgi_info->to_cgi, session->body, session->content_length);
+				if (write_buffer(cgi_info->to_cgi, session->body, session->content_length) == -1) {
+					close(cgi_info->to_cgi);
+					close(cgi_info->from_cgi);
+					close(cgi_info->cgi_error);
+					return -1;
+				}
 			}
 	}
 
@@ -377,8 +390,10 @@ t_cgi_result read_from_cgi_process(t_session *session, t_cgi_info *cgi_info) {
 			} else {
 				read_again = true;
 			}
-//		} else if (bytes_read == 0) {
-//			result = cgi_END_OF_DATA;
+/*
+		} else if (bytes_read == 0) {
+			result = cgi_END_OF_DATA;
+*/
 		} else if (bytes_read > 0) {
 			cgi_info->error_len += bytes_read;
 		}
@@ -399,7 +414,7 @@ int connect_to_fcgi_server(t_connect_to *connect_to) {
 
 	if ((hostinfo = gethostbyname(connect_to->host)) != NULL) {
 		if ((sock = socket(AF_INET, SOCK_STREAM, 0)) > 0) {
-			bzero(&saddr, sizeof(struct sockaddr_in));
+			memset(&saddr, 0, sizeof(struct sockaddr_in));
 			saddr.sin_family = AF_INET;
 			saddr.sin_port = htons(connect_to->port);
 			memcpy(&saddr.sin_addr.s_addr, hostinfo->h_addr, 4);
@@ -417,7 +432,7 @@ int connect_to_fcgi_server(t_connect_to *connect_to) {
 	return sock;
 }
 
-int	send_fcgi_request(t_session *session, int sock) {
+int send_fcgi_request(t_session *session, int sock) {
 	t_fcgi_buffer fcgi_buffer;
 
 	fcgi_buffer.sock = sock;
