@@ -3,6 +3,7 @@
 
 #include "config.h"
 #include <stdbool.h>
+#include "global.h"
 #include "serverconfig.h"
 #include "session.h"
 
@@ -15,18 +16,18 @@ int  add_client(t_session *session);
 int  mark_client_for_removal(t_session *session, int delay);
 void check_delayed_remove_timers(t_config *config);
 int  remove_client(t_session *session, bool free_session);
-int  connection_allowed(long ip, int maxperip, int maxtotal);
+int  connection_allowed(t_ipv4 ip, int maxperip, int maxtotal);
 int  disconnect_clients(t_config *config);
 bool client_is_flooding(t_session *session);
 void check_flooding(t_config *config);
 
 int  kick_client(int id);
-int  kick_ip(long ip);
+int  kick_ip(t_ipv4 ip);
 
-int  ban_ip(long ip, int timer, bool kick_on_ban);
-int  reban_ip(long ip);
+int  ban_ip(t_ipv4 ip, int timer, bool kick_on_ban);
+int  reban_ip(t_ipv4 ip);
 void check_ban_list(t_config *config);
-int  unban_ip(long ip);
+int  unban_ip(t_ipv4 ip);
 
 #ifdef HAVE_COMMAND
 void print_client_list(FILE *fp);
