@@ -83,7 +83,7 @@ int add_admin(int sock) {
 	if ((new = (t_admin*)malloc(sizeof(t_admin))) != NULL) {
 		if ((new->fp = fdopen(sock, "r+")) != NULL) {
 			fprintf(new->fp, "\n\033[02;31mHiawatha v"VERSION" CommandChannel\033[00m\n");
-			fprintf(new->fp, "Password: \0337\033[00;30;40m"); // Save cursor position and change color to black
+			fprintf(new->fp, "Password: \0337\033[00;30;40m"); /* Save cursor position and change color to black */
 			fflush(new->fp);
 
 			new->next = adminlist;
@@ -171,7 +171,7 @@ t_admin *next_admin(void) {
 
 /* Check administratos (only auto-logout timers for now).
  */
-void check_adminlist(void) {
+void check_admin_list(void) {
 	t_admin *admin, *prev_admin = NULL, *next_admin;
 
 	admin = adminlist;
@@ -349,7 +349,7 @@ static int run_command(char *line, FILE *fp, t_config *config) {
  */
 int handle_admin(t_admin *admin, t_config *config) {
 	int retval = cc_OKE;
-	char line[MAX_CMD_SIZE + 1], *pwd, encrypted[33]; //, salt[3];
+	char line[MAX_CMD_SIZE + 1], *pwd, encrypted[33];
 
 	if (fgets(line, MAX_CMD_SIZE, admin->fp) != NULL) {
 		line[MAX_CMD_SIZE] = '\0';
@@ -366,7 +366,7 @@ int handle_admin(t_admin *admin, t_config *config) {
 		} else if (admin->authenticated) {
 			retval = run_command(remove_spaces(line), admin->fp, config);
 		} else {
-			fprintf(admin->fp, "\0338\033[K\0338\n"); // Restore cursor position and color and erase to end of line
+			fprintf(admin->fp, "\0338\033[K\0338\n"); /* Restore cursor position and color and erase to end of line */
 			pwd = remove_spaces(line);
 			md5_hash(pwd, strlen(pwd), encrypted);
 			if ((admin->authenticated = (strcmp(encrypted, config->command_port->binding_id) == 0)) == false) {
