@@ -38,6 +38,7 @@
 #include "target.h"
 #include "log.h"
 #include "envir.h"
+#include "global.h"
 #ifdef HAVE_COMMAND
 #include "command.h"
 #endif
@@ -53,6 +54,7 @@
 
 #define REQUEST_BUFFER_CHUNK  4096
 #define MAIN_LOOP_TIMER          1
+#define PTHREAD_STACK_SIZE     512 * KILOBYTE
 
 #define fe_MAX_REQUESTSIZE      -2
 #define fe_TIMEOUT              -3
@@ -100,10 +102,8 @@ char *version_string = "Hiawatha v"VERSION
 ;
 
 #ifdef DEBUG
-
 /* timer functions for debugging
  */
-
 double t1, t2;
 struct timeval tp;
 
@@ -118,7 +118,6 @@ void stop_clock(void) {
 	t2 = (double)tp.tv_sec + (1.e-6)*tp.tv_usec;
 	fprintf(stderr, "(%5d)  time: %f\n", (int)getpid(), t2 - t1);
 }
-
 #endif
 
 /* Create all logfiles with the right ownership and accessrights
@@ -201,7 +200,7 @@ int handle_error(t_session *session, int error_code) {
 		free(session->file_on_disk);
 	}
 	length = session->host->website_root_len;
-	if ((session->file_on_disk = (char*)malloc(length + strlen(error_handler) + 4)) == NULL) { // + 3 for .gz (gzip encoding)
+	if ((session->file_on_disk = (char*)malloc(length + strlen(error_handler) + 4)) == NULL) { /* + 3 for .gz (gzip encoding) */
 		result = 500;
 	} else {
 		memcpy(session->file_on_disk, session->host->website_root, length + 1);
@@ -439,7 +438,7 @@ int fetch_request(t_session *session) {
 						}
 					} else {
 						if (session->bytes_in_buffer >= header_length + content_length) {
-							// Received a complete request
+							/* Received a complete request */
 							break;
 						}
 					}
@@ -510,7 +509,7 @@ int fetch_request(t_session *session) {
 						break;
 					default:
 						if (store_on_disk) {
-							// Write to file on disk
+							/* Write to file on disk */
 							write_bytes = bytes_read;
 							if (session->uploaded_size + bytes_read > content_length) {
 								write_bytes -= ((session->uploaded_size + bytes_read) - content_length);
@@ -531,7 +530,7 @@ int fetch_request(t_session *session) {
 								keep_reading = false;
 							}
 						} else {
-							// Read into memory
+							/* Read into memory */
 							session->bytes_in_buffer += bytes_read;
 							*(session->request + session->bytes_in_buffer) = '\0';
 
@@ -546,6 +545,7 @@ int fetch_request(t_session *session) {
 	} while (keep_reading);
 
 	if (upload_handle != -1) {
+		fsync(upload_handle);
 		close(upload_handle);
 	}
 
@@ -555,6 +555,7 @@ int fetch_request(t_session *session) {
 
 #ifdef DEBUG
 	set_client_status(session, NULL);
+	fprintf(stderr, "Request size: %ld + %ld\n", header_length, content_length);
 #endif
 
 	return result;
@@ -696,7 +697,7 @@ int uri_to_path(t_session *session) {
 					return 500;
 				}
 			} else {
-				// uri is '/~/...'
+				/* uri is '/~/...' */
 				return 404;
 			}
 		}
@@ -724,7 +725,7 @@ int uri_to_path(t_session *session) {
 		length = strlen(alias->value);
 	}
 	length += session->uri_len + strlen(session->host->start_file);
-	if ((session->file_on_disk = (char*)malloc(length + 4)) == NULL) { // + 3 for '.gz' (gzip encoding)
+	if ((session->file_on_disk = (char*)malloc(length + 4)) == NULL) { /* + 3 for '.gz' (gzip encoding) */
 		return 500;
 	}
 
@@ -825,6 +826,7 @@ int serve_client(t_session *session) {
 	char *search, *conffile, *qmark;
 	t_host *host_record;
 	bool file_not_found = false;
+	t_fsbool is_dir;
 #ifdef HAVE_REWRITE
 	int i;
 	char *new_uri;
@@ -836,18 +838,18 @@ int serve_client(t_session *session) {
 		return result;
 	}
 
-	//session->host = session->config->first_host;
 	if (session->hostname != NULL) {
 		if ((host_record = get_hostrecord(session->config->first_host, session->hostname, session->binding)) != NULL) {
 			session->host = host_record;
 		}
 	}
+	session->host->access_time = time(NULL);
 
 #ifdef HAVE_REWRITE
 	/* URL rewrite
 	 */
 	for (i = 0; i < session->host->rewrite_rules.size; i++) {
-		if (rewrite_url(session->uri, session->host->rewrite_rules.item[i], session->config->url_rewrite, &new_uri) == -1) {
+		if ((result = rewrite_url(session->uri, session->host->rewrite_rules.item[i], session->config->url_rewrite, &new_uri)) == RW_ERROR) {
 			return 500;
 		}
 		if (new_uri != NULL) {
@@ -860,6 +862,10 @@ int serve_client(t_session *session) {
 #endif
 			session->uri = new_uri;
 		}
+
+		if (result == RW_DENY_ACCESS) {
+			return 403;
+		}
 	}
 #endif
 
@@ -972,6 +978,14 @@ int serve_client(t_session *session) {
 				if (get_path_info(session) == -1) {
 					return 500;
 				}
+			} else if ((search = strrchr(session->file_on_disk, '/')) != NULL) {
+				*search = '\0';
+				is_dir = is_directory(session->file_on_disk);
+				*search = '/';
+
+				if (is_dir == no) {
+					return 404;
+				}
 			}
 			break;
 		case no_access:
@@ -986,11 +1000,13 @@ int serve_client(t_session *session) {
 				}
 			}
 
-			if ((session->put_method == false) && (session->accept_gzip == false) && (session->host->fast_cgi.size == 0)) {
-				return 404;
+			if ((session->accept_gzip == false) && (session->put_method == false)) {
+				if (session->host->fast_cgi.size == 0) {
+					return 404;
+				} else {
+					file_not_found = true;
+				}
 			}
-
-			file_not_found = true;
 	}
 
 	length = strlen(session->file_on_disk);
@@ -1006,7 +1022,7 @@ int serve_client(t_session *session) {
 
 	check_target_is_cgi(session);
 
-	if (file_not_found && (session->put_method == false) && (session->cgi_type != fastcgi)) {
+	if (file_not_found && (session->cgi_type != fastcgi)) {
 		return 404;
 	}
 
@@ -1024,7 +1040,7 @@ void handle_request_result(t_session *session, int result) {
 
 	switch (result) {
 		case fe_MAX_REQUESTSIZE:
-			log_error(session, "Maximum request size reached");
+			log_system(session, "Maximum request size reached");
 			send_code(session, 413);
 			if ((session->config->ban_on_max_request_size > 0) && ip_allowed(session->ip_address, session->config->banlist_mask)) {
 				ban_ip(session->ip_address, session->config->ban_on_max_request_size, session->config->kick_on_ban);
@@ -1151,7 +1167,7 @@ void connection_handler(t_session *session) {
 		do {
 			result = serve_client(session);
 			handle_request_result(session, result);
-			send_buffer(session, NULL, 0); // Flush the output-buffer
+			send_buffer(session, NULL, 0); /* Flush the output-buffer */
 
 			reset_session(session);
 
@@ -1171,14 +1187,13 @@ void connection_handler(t_session *session) {
 	} else {
 		close(session->client_socket);
 	}
-	log_close(session->config->first_host);
 
 	if (session->config->reconnect_delay > 0) {
 		mark_client_for_removal(session, session->config->reconnect_delay);
 	} else {
 		remove_client(session, true);
 	}
-	// Client session ends here.
+	/* Client session ends here */
 }
 
 /* Signal handlers
@@ -1254,7 +1269,7 @@ int bind_sockets(t_binding *binding) {
 			return -1;
 		}
 
-		bzero(&saddr, sizeof(struct sockaddr_in));
+		memset(&saddr, 0, sizeof(struct sockaddr_in));
 		saddr.sin_family = AF_INET;
 		saddr.sin_port = htons(binding->port);
 		saddr.sin_addr.s_addr = binding->interface;
@@ -1298,7 +1313,7 @@ int accept_connection(t_binding *binding, t_config *config) {
 	init_session(session);
 
 	size = sizeof(struct sockaddr_in);
-	memset((void*)&caddr, 0, size);
+	memset((void*)&caddr, 0, (size_t)size);
 	if ((session->client_socket = accept(binding->socket, (struct sockaddr*)&caddr, &size)) == -1) {
 		free(session);
 		if (errno != EINTR) {
@@ -1319,10 +1334,16 @@ int accept_connection(t_binding *binding, t_config *config) {
 			}
 		}
 
-		if (add_client(session) == 0) {
-			pthread_attr_init(&child_attr);
-			pthread_attr_setdetachstate(&child_attr, PTHREAD_CREATE_DETACHED);
+		if (pthread_attr_init(&child_attr) != 0) {
+			log_system(session, "pthread init error");
+		} else if (pthread_attr_setdetachstate(&child_attr, PTHREAD_CREATE_DETACHED) != 0) {
+			log_system(session, "pthread set detach state error");
+		} else if (pthread_attr_setstacksize(&child_attr, PTHREAD_STACK_SIZE) != 0) {
+			log_system(session, "pthread set stack size error");
+		} else if (add_client(session) == 0) {
 			if (pthread_create(&child_thread, &child_attr, (void*)connection_handler, (void*)session) == 0) {
+				/* Thread started
+				 */
 				kick_client = false;
 			} else {
 				remove_client(session, false);
@@ -1382,7 +1403,9 @@ int run_server(t_settings *settings) {
 	mode_t             access_rights;
 
 	signal(SIGPIPE, SIG_IGN);
+#ifndef OPENBSD
 	signal(SIGCHLD, SIG_IGN);
+#endif
 	signal(SIGTSTP, SIG_IGN);
 	signal(SIGABRT, SIG_IGN);
 	signal(SIGQUIT, SIG_IGN);
@@ -1446,7 +1469,7 @@ int run_server(t_settings *settings) {
 	}
 #endif
 
-	/* The upload directory for PUT requests
+	/* Create the upload directory for PUT requests
 	 */
 	if (config->create_upload_dir) {
 		if (mkdir(config->upload_directory, 0733) == -1) {
@@ -1592,21 +1615,32 @@ int run_server(t_settings *settings) {
 			case -1:
 				if (errno != EINTR) {
 					log_string(config->system_logfile, "Fatal error selecting connection");
-					usleep(1000);
+					sleep(1);
 				}
 				break;
 			case 0:
 				select_timeout.tv_sec = MAIN_LOOP_TIMER;
 				select_timeout.tv_usec = 0;
 
+				/* Client checks
+				 */
 				check_ban_list(config);
 				check_delayed_remove_timers(config);
+				/* FastCGI check
+				 */
 				check_load_balancer(config);
+				/* AccessLogfile check
+				 */
+				log_close(config->first_host, false);
 #ifdef HAVE_CACHE
+				/* Cache check
+				 */
 				check_cache();
 #endif
 #ifdef HAVE_COMMAND
-				check_adminlist();
+				/* CommandChannel check
+				 */
+				check_admin_list();
 #endif
 				switch (received_signal) {
 					case rs_NONE:
@@ -1619,6 +1653,7 @@ int run_server(t_settings *settings) {
 						received_signal = rs_NONE;
 						break;
 					case rs_TOUCH_LOGFILES:
+						log_close(config->first_host, true);
 						touch_logfiles(config, NULL);
 						received_signal = rs_NONE;
 						break;
@@ -1626,7 +1661,7 @@ int run_server(t_settings *settings) {
 				break;
 			default:
 #ifdef HAVE_COMMAND
-				// Connected admins
+				/* Connected admins */
 				admin = first_admin();
 				while (admin != NULL) {
 					if (FD_ISSET(admin->socket, &read_fds)) {
@@ -1638,7 +1673,7 @@ int run_server(t_settings *settings) {
 				}
 #endif
 
-				// HTTP(S) port
+				/* HTTP(S) port */
 				binding = config->binding;
 				while (binding != NULL) {
 					if (FD_ISSET(binding->socket, &read_fds)) {
@@ -1653,12 +1688,12 @@ int run_server(t_settings *settings) {
 				}
 
 #ifdef HAVE_COMMAND
-				// Command port
+				/* Command port */
 				binding = config->command_port;
 				while (binding != NULL) {
 					if (FD_ISSET(binding->socket, &read_fds)) {
 						size = sizeof(struct sockaddr_in);
-						memset((void*)&caddr, 0, size);
+						memset((void*)&caddr, 0, (size_t)size);
 						if ((admin_socket = accept(binding->socket, (struct sockaddr*)&caddr, &size)) == -1) {
 							if (errno != EINTR) {
 								log_string(config->system_logfile, "Fatal error accepting CommandChannel connection");
@@ -1683,17 +1718,7 @@ int run_server(t_settings *settings) {
 #ifdef HAVE_COMMAND
 	disconnect_admins();
 #endif
-
-	binding = config->binding;
-	while (binding != NULL) {
-		close(binding->socket);
-#ifdef HAVE_SSL
-		if (binding->use_ssl) {
-			ssl_cleanup(binding->ssl_context);
-		}
-#endif
-		binding = binding->next;
-	}
+	close_bindings(config->binding);
 
 #ifdef HAVE_COMMAND
 	binding = config->command_port;
