@@ -224,11 +224,11 @@ static bool digest_http_authentication(t_session *session, char *auth_str) {
 	}
 
 	if (session->vars != NULL) {
-		if (strncmp(session->uri, uri, strlen(session->uri)) != 0) {
+		if (strncmp(session->uri, uri, session->uri_len) != 0) {
 			return false;
-		} else if (*(uri + strlen(session->uri)) != '?') {
+		} else if (*(uri + session->uri_len) != '?') {
 			return false;
-		} else if (strcmp(session->vars, uri + strlen(session->uri) + 1) != 0) {
+		} else if (strcmp(session->vars, uri + session->uri_len + 1) != 0) {
 			return false;
 		}
 	} else if (strcmp(session->uri, uri) != 0) {
@@ -266,8 +266,8 @@ static bool digest_http_authentication(t_session *session, char *auth_str) {
 
 			/* Password match?
 			 */
-			if ((retval = (strcmp(result, response) == 0))) {
-				session->remote_user = strdup(username);
+			if (strcmp(result, response) == 0) {
+				retval = ((session->remote_user = strdup(username)) != NULL);
 			}
 		} while (false);
 		free(passwd);
