@@ -2,6 +2,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "global.h"
 #include "libstr.h"
 #include "liblist.h"
 
@@ -55,7 +56,7 @@ int parse_ip(char *line, t_ipv4 *l_ip) {
 	return 0;
 }
 
-// --< charlist >--------------------------------------------------------------
+/*---< charlist >-------------------------------------------------------------*/
 
 void init_charlist(t_charlist *list) {
 	list->size = 0;
@@ -137,7 +138,7 @@ void remove_charlist(t_charlist *list) {
 	}
 }
 
-// --< accesslist >------------------------------------------------------------
+/*---< accesslist >-----------------------------------------------------------*/
 
 /* Parse a list of access levels.
  */
@@ -249,18 +250,18 @@ t_access ip_allowed(t_ipv4 ip, t_accesslist *list) {
 	return unknown;
 }
 
-// --< key/valuelist >---------------------------------------------------------
+/*---< key/valuelist >-------------------------------------------------------*/
 
 /* Parse a list of key/value combinations.
  */
-int parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *seperator) {
+int parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *delimiter) {
 	char *value;
 	t_keyvalue *prev;
 
 	if (line != NULL) {
-		if ((value = strstr(line, seperator)) != NULL) {
+		if ((value = strstr(line, delimiter)) != NULL) {
 			*value = '\0';
-			value += strlen(seperator);
+			value += strlen(delimiter);
 
 			prev = *kvlist;
 			if ((*kvlist = (t_keyvalue*)malloc(sizeof(t_keyvalue))) == NULL) {
@@ -301,7 +302,7 @@ t_keyvalue *remove_keyvaluelist(t_keyvalue *list) {
 	return NULL;
 }
 
-// --< temporary data >-------------------------------------------------------
+/*---< temporary data >------------------------------------------------------*/
 
 int register_tempdata(t_tempdata **tempdata, void *data, t_tempdata_type type) {
 	t_tempdata *tdata;
