@@ -44,20 +44,20 @@ typedef struct type_tempdata {
 void sfree(void* ptr);
 
 void init_charlist(t_charlist *list);
-int parse_charlist(char *value, t_charlist *list);
+int  parse_charlist(char *value, t_charlist *list);
 void copy_charlist(t_charlist *dest, t_charlist *src);
 bool in_charlist(char *item, t_charlist *list);
 void remove_charlist(t_charlist *list);
 
-int parse_ip(char *line, t_ipv4 *l_ip);
+int  parse_ip(char *line, t_ipv4 *l_ip);
 t_accesslist *parse_accesslist(char *line, bool pwd_allowed, t_accesslist *list);
 t_accesslist *remove_accesslist(t_accesslist *list);
 t_access ip_allowed(t_ipv4 ip, t_accesslist *list);
 
-int parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *seperator);
+int  parse_keyvaluelist(char *line, t_keyvalue **kvlist, char *delimiter);
 t_keyvalue *remove_keyvaluelist(t_keyvalue *list);
 
-int register_tempdata(t_tempdata **tempdata, void *data, t_tempdata_type type);
+int  register_tempdata(t_tempdata **tempdata, void *data, t_tempdata_type type);
 void remove_tempdata(t_tempdata *tempdata);
 
 #endif
