@@ -489,12 +489,14 @@ char *file_extension(char *filename) {
 
 int str_replace(char *src, char *from, char *to, char **dst) {
 	char *pos;
-	int replaced = 0;
+	int replaced = 0, len_from, len_to;
+
+	len_from = strlen(from);
+	len_to   = strlen(to);
 
-	*dst = NULL;
 	if ((src != NULL) && (from != NULL) && (to != NULL)) {
 		while ((pos = strstr(src, from)) != NULL) {
-			if ((*dst = (char*)malloc(strlen(src) - strlen(from) + strlen(to) + 1)) == NULL) {
+			if ((*dst = (char*)malloc(strlen(src) - len_from + len_to + 1)) == NULL) {
 				if (*dst != NULL) {
 					free(*dst);
 				}
@@ -502,7 +504,7 @@ int str_replace(char *src, char *from, char *to, char **dst) {
 			}
 
 			*pos = '\0';
-			sprintf(*dst, "%s%s%s", src, to, pos + strlen(from));
+			sprintf(*dst, "%s%s%s", src, to, pos + len_from);
 
 			if (replaced > 0) {
 				free(src);
