@@ -16,10 +16,15 @@
 #define TIMESTAMP_SIZE    40
 #define IP_ADDRESS_SIZE   20
 
+#define MAX_LOG_DELAY_TIMER 10
+#define LOGFILE_OPEN_TIME   30
+
 pthread_mutex_t accesslog_mutex;
+static int delay_timer = 0;
 
 void init_log_module(void) {
 	pthread_mutex_init(&accesslog_mutex, NULL);
+	delay_timer = 0;
 }
 
 /* Write a timestamp to a logfile.
@@ -47,17 +52,19 @@ static void print_ip_address(char *str, t_ipv4 ip_address) {
 static char *secure_string(char *str) {
 	char *c = str;
 
-	while (*c != '\0') {
-		if (*c == '\27') {
-			*c = ' ';
+	if (str != NULL) {
+		while (*c != '\0') {
+			if (*c == '\27') {
+				*c = ' ';
+			}
+			c++;
 		}
-		c++;
 	}
 
 	return str;
 }
 
-// ---< Main log functions >------------------------------------------
+/*---< Main log functions >------------------------------------------*/
 
 /* Log the Hiawatha process ID.
  */
@@ -120,7 +127,7 @@ void log_error(t_session *session, char *mesg) {
  */
 void log_request(t_session *session, int code) {
 	char str[BUFFER_SIZE + 1], timestamp[TIMESTAMP_SIZE], ip_address[IP_ADDRESS_SIZE];
-	char *user, *field;
+	char *user, *field, *uri, *vars, *path_info;
 	t_headerfield *headerfield;
 	int offset, len;
 	time_t t;
@@ -129,29 +136,45 @@ void log_request(t_session *session, int code) {
 	if (ip_allowed(session->ip_address, session->config->logfile_mask) != deny) {
 		str[BUFFER_SIZE] = '\0';
 
+#ifdef HAVE_REWRITE
+		if (session->script_url == NULL) {
+#endif
+			uri = secure_string(session->uri);
+			path_info = secure_string(session->path_info);
+			vars = secure_string(session->vars);
+#ifdef HAVE_REWRITE
+
+		} else {
+			uri = secure_string(session->script_url);
+			path_info = NULL;
+			vars = NULL;
+		}
+#endif
+
+		if ((user = session->remote_user) == NULL) {
+			user = "";
+		} else {
+			user = secure_string(user);
+		}
+
 		if (session->config->log_format == hiawatha) {
 			/* Hiawatha log format
 			 */
-			if ((user = session->remote_user) == NULL) {
-				user = "";
-			} else {
-				user = secure_string(user);
-			}
 
 			print_ip_address(str, session->ip_address);
 			offset = strlen(str);
 			print_timestamp(str + offset);
 			offset += strlen(str + offset);
 
-			snprintf(str + offset, BUFFER_SIZE - offset, "%d|%lld|%s|%s %s", code, (long long)session->bytes_sent, user, secure_string(session->method), secure_string(session->uri));
+			snprintf(str + offset, BUFFER_SIZE - offset, "%d|%lld|%s|%s %s", code, (long long)session->bytes_sent, user, secure_string(session->method), uri);
 			offset += strlen(str + offset);
 
-			if ((offset < BUFFER_SIZE) && (session->path_info != NULL)) {
-				snprintf(str + offset, BUFFER_SIZE - offset, "/%s", secure_string(session->path_info));
+			if ((offset < BUFFER_SIZE) && (path_info != NULL)) {
+				snprintf(str + offset, BUFFER_SIZE - offset, "/%s", path_info);
 				offset += strlen(str + offset);
 			}
-			if ((offset < BUFFER_SIZE) && (session->vars != NULL)) {
-				snprintf(str + offset, BUFFER_SIZE - offset, "?%s", secure_string(session->vars));
+			if ((offset < BUFFER_SIZE) && (vars != NULL)) {
+				snprintf(str + offset, BUFFER_SIZE - offset, "?%s", vars);
 				offset += strlen(str + offset);
 			}
 
@@ -163,7 +186,7 @@ void log_request(t_session *session, int code) {
 			if (offset < BUFFER_SIZE) {
 				headerfield = session->headerfields;
 				while (headerfield != NULL) {
-					if (strncmp("Authorization:", headerfield->data, 14) != 0) {
+					if ((strncasecmp("Cookie:", headerfield->data, 7) != 0) && (strncasecmp("Authorization:", headerfield->data, 14) != 0)) {
 						snprintf(str + offset, BUFFER_SIZE - offset, "|%s", secure_string(headerfield->data));
 						if ((offset += strlen(str + offset)) >= BUFFER_SIZE) {
 							break;
@@ -185,20 +208,14 @@ void log_request(t_session *session, int code) {
 			timestamp[TIMESTAMP_SIZE - 1] = '\0';
 			strftime(timestamp, TIMESTAMP_SIZE - 1, "%d/%b/%Y:%T %z", s);
 
-			if ((user = session->remote_user) == NULL) {
-				user = "-";
-			} else {
-				user = secure_string(user);
-			}
-
-			snprintf(str, BUFFER_SIZE, "%s - %s [%s] \"%s %s", ip_address, user, timestamp, secure_string(session->method), secure_string(session->uri));
+			snprintf(str, BUFFER_SIZE, "%s - %s [%s] \"%s %s", ip_address, user, timestamp, secure_string(session->method), uri);
 			offset = strlen(str);
-			if ((offset < BUFFER_SIZE) && (session->path_info != NULL)) {
-				snprintf(str + offset, BUFFER_SIZE - offset, "/%s", secure_string(session->path_info));
+			if ((offset < BUFFER_SIZE) && (path_info != NULL)) {
+				snprintf(str + offset, BUFFER_SIZE - offset, "/%s", path_info);
 				offset += strlen(str + offset);
 			}
-			if ((offset < BUFFER_SIZE) && (session->vars != NULL)) {
-				snprintf(str + offset, BUFFER_SIZE - offset, "?%s", secure_string(session->vars));
+			if ((offset < BUFFER_SIZE) && (vars != NULL)) {
+				snprintf(str + offset, BUFFER_SIZE - offset, "?%s", vars);
 				offset += strlen(str + offset);
 			}
 			if (offset < BUFFER_SIZE) {
@@ -301,14 +318,24 @@ void log_file_error(t_session *session, char *file, char *mesg) {
 
 /* Close open accesslogfiles.
  */
-void log_close(t_host *host) {
-	pthread_mutex_lock(&accesslog_mutex);
-	while (host != NULL) {
-		if (*(host->access_fp) != NULL) {
-			fclose(*(host->access_fp));
-			*(host->access_fp) = NULL;
+void log_close(t_host *host, bool force_close) {
+	time_t now;
+
+	if ((++delay_timer >= MAX_LOG_DELAY_TIMER) || force_close) {
+		delay_timer = 0;
+		now = time(NULL);
+
+		pthread_mutex_lock(&accesslog_mutex);
+		while (host != NULL) {
+			if ((now >= host->access_time + LOGFILE_OPEN_TIME) || force_close) {
+				if (*(host->access_fp) != NULL) {
+					fflush(*(host->access_fp));
+					fclose(*(host->access_fp));
+					*(host->access_fp) = NULL;
+				}
+			}
+			host = host->next;
 		}
-		host = host->next;
+		pthread_mutex_unlock(&accesslog_mutex);
 	}
-	pthread_mutex_unlock(&accesslog_mutex);
 }
