@@ -13,6 +13,6 @@ void log_request(t_session *session, int code);
 void log_garbage(t_session *session);
 void log_unban(char *logfile, t_ipv4 ip_address, unsigned long connect_attempts);
 void log_file_error(t_session *session, char *file, char *mesg);
-void log_close(t_host *host);
+void log_close(t_host *host, bool force_close);
 
 #endif
