@@ -12,6 +12,7 @@
 #include "userconfig.h"
 
 #define MAX_CONFIG_LINE 1024
+#define DELIMITER ','
 
 /* Type definitions
  */
@@ -104,7 +105,7 @@ int parse_line(t_config *config, char *line) {
 
 			/* PHP executable
 			 */
-			if (split_string(value, &item, &value, ',') == -1) {
+			if (split_string(value, &item, &value, DELIMITER) == -1) {
 				return -1;
 			}
 			if ((pipe = strchr(item, '|')) != NULL) {
@@ -123,7 +124,7 @@ int parse_line(t_config *config, char *line) {
 
 			/* Binding
 			 */
-			if (split_string(value, &item, &value, ',') == -1) {
+			if (split_string(value, &item, &value, DELIMITER) == -1) {
 				return -1;
 			}
 			if ((new->binding = strdup(item)) == NULL) {
