@@ -51,6 +51,7 @@ static t_host *new_host(void) {
 		host->access_logfile     = LOG_DIR"/access.log";
 		host->access_fileptr     = NULL;
 		host->access_fp          = &(host->access_fileptr);
+		host->access_time        = 0;
 		host->error_logfile      = LOG_DIR"/error.log";
 		init_charlist(&(host->hostname));
 		host->user_websites      = false;
@@ -73,8 +74,7 @@ static t_host *new_host(void) {
 		init_charlist(&(host->rewrite_rules));
 #endif
 		host->wrap_cgi           = NULL;
-		host->groups.number      = 0;
-		host->groups.array       = NULL;
+		init_groups(&(host->groups));
 		init_charlist(&(host->volatile_object));
 		init_charlist(&(host->image_referer));
 		host->imgref_replacement = NULL;
@@ -114,8 +114,7 @@ static t_directory *new_directory(void) {
 		directory->access_list         = NULL;
 		directory->alter_list          = NULL;
 		directory->alter_fmode         = 0;
-		directory->groups.number       = 0;
-		directory->groups.array        = NULL;
+		init_groups(&(directory->groups));
 		directory->passwordfile        = NULL;
 		directory->groupfile           = NULL;
 		init_charlist(&(directory->required_group));
@@ -221,8 +220,7 @@ t_config *default_config(void) {
 		config->server_uid         = (uid_t)ID_NOBODY;
 		config->server_gid         = (gid_t)ID_NOBODY;
 		config->server_string      = "Hiawatha v"VERSION;
-		config->groups.number      = 0;
-		config->groups.array       = NULL;
+		init_groups(&(config->groups));
 #ifdef HAVE_SSL
 		config->server_key         = CONFIG_DIR"/serverkey.pem";
 		config->ca_cert            = NULL;
@@ -275,8 +273,8 @@ t_config *default_config(void) {
 #endif
 
 #ifdef HAVE_CACHE
-		config->cache_size         = 10485760;  //  10MB
-		config->cache_max_filesize = 262144;    // 256Kb
+		config->cache_size         = 10485760;  /*  10MB */
+		config->cache_max_filesize = 262144;    /* 256Kb */
 		config->cache_min_filesize = 0;
 #endif
 
@@ -376,7 +374,7 @@ static int parse_yesno(char *yesno, bool *result) {
 	return 0;
 }
 
-int parse_credentialfiles(char *line, t_auth_method *auth_method, char **pwdfile, char **groupfile) {
+static int parse_credentialfiles(char *line, t_auth_method *auth_method, char **pwdfile, char **groupfile) {
 	char *file, *group;
 
 	split_string(line, &line, &group, ',');
@@ -414,7 +412,7 @@ int parse_credentialfiles(char *line, t_auth_method *auth_method, char **pwdfile
 	return 0;
 }
 
-bool replace_variables(char **line) {
+static bool replace_variables(char **line) {
 	bool replaced = false;
 	t_keyvalue *variable;
 	char *new;
@@ -434,6 +432,71 @@ bool replace_variables(char **line) {
 	return replaced;
 }
 
+#ifdef CYGWIN
+static int fix_windows_path(char *value, char *key) {
+	char *pos;
+	size_t len;
+
+	if (key != NULL) {
+		if (strcmp(key, "setenv") == 0) {
+			return 0;
+		}
+	}
+
+	if (value == NULL) {
+		return -1;
+	} else if (strlen(value) + 40 > MAX_LENGTH_CONFIGLINE) {
+		return -1;
+	}
+	if ((pos = strstr(value, ":\\")) == NULL) {
+		return 0;
+	}
+	pos--;
+	if (pos > value) {
+		if ((*(pos - 1) != ':') && (*(pos - 1) != ',')) {
+			return 0;
+		}
+	} else if (pos != value) {
+		return 0;
+	}
+	if ((*pos >= 'A') && (*pos <= 'Z')) {
+		*pos += 32;
+	} else if ((*pos < 'a') || (*pos > 'z')) {
+		return 0;
+	}
+	len = strlen(pos) - 2;
+	memmove(pos + 12, pos + 3, len);
+	*(pos + 10) = *pos;
+	*(pos + 11) = '/';
+	memcpy(pos, "/cygdrive/", 10);
+
+	pos = pos + 12;
+	while (*pos != '\0') {
+		if (*pos == '\\') {
+			*pos = '/';
+		} else if ((*pos == ':') || (*pos == ',')) {	
+			break;
+		}
+		pos++;
+	}
+
+	return 1;
+}
+#endif
+
+void close_bindings(t_binding *binding) {
+	while (binding != NULL) {
+#ifdef HAVE_SSL
+		if (binding->use_ssl) {
+			ssl_cleanup(binding->ssl_context);
+		}
+#endif
+		close(binding->socket);
+
+		binding = binding->next;
+	}
+}
+
 /******************************************************************************
  *
  * Check configuration for validity
@@ -812,7 +875,7 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				throt->next = NULL;
 
 				if ((throt->filetype = strlower(strdup(rest))) != NULL) {
-					throt->upload_speed = speed << 10; // convert to kilobytes
+					throt->upload_speed = speed << 10; /* convert to kilobytes */
 					return true;
 				}
 			}
@@ -1341,7 +1404,7 @@ static bool fcgi_server_setting(char *key, char *value, t_fcgi_server *fcgi_serv
 			return true;
 		}
 	} else if (strcmp(key, "sessiontimeout") == 0) {
-		if ((fcgi_server->session_timeout = 60 * str2int(value)) >= 0) {
+		if ((fcgi_server->session_timeout = MINUTE * str2int(value)) >= 0) {
 			return true;
 		}
 	}
@@ -1554,9 +1617,14 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 					}
 				} else if (strcmp(key, "include") == 0) {
 					value = key + 8;
-					variables_replaced = replace_variables(&value);
-					if (section == none) {
-						if (including == false) {
+#ifdef CYGWIN
+					if (fix_windows_path(value, NULL) == -1) {
+						retval = counter;
+					} else
+#endif
+					{
+						variables_replaced = replace_variables(&value);
+						if ((section == none) && (including == false)) {
 							including = true;
 							switch (is_directory(value)) {
 								case error:
@@ -1575,44 +1643,51 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 						} else {
 							retval = counter;
 						}
-					} else {
-						retval = counter;
 					}
 				} else if (strlen(value) > 0) {
-					variables_replaced = replace_variables(&value);
-					do {
-						if (section == none) {
-							if (system_setting(key, value, config)) {
-								break;
-							}
-						}
-						if ((section == none) || (section == virtual_host)) {
-							if (host_setting(key, value, current_host)) {
-								break;
-							} else if (user_setting(key, value, current_host, NULL)) {
-								break;
-							}
-						} else if (section == directory) {
-							if (directory_setting(key, value, current_directory)) {
-								break;
-							}
-						} else if (section == binding) {
-							if (binding_setting(key, value, current_binding)) {
-								break;
-							}
-						} else if (section == fcgi_server) {
-							if (fcgi_server_setting(key, value, current_fcgi_server)) {
-								break;
-							}
-#ifdef HAVE_REWRITE
-						} else if (section == url_rewrite) {
-							if (url_rewrite_setting(key, value, current_rewrite)) {
-								break;
-							}
-#endif
-						}
+#ifdef CYGWIN
+					if (strlen(key) > 25) {
 						retval = counter;
-					} while (false);
+					} else if (fix_windows_path(value, key) == -1) {
+						retval = counter;
+					} else
+#endif
+					{
+						variables_replaced = replace_variables(&value);
+						do {
+							if (section == none) {
+								if (system_setting(key, value, config)) {
+									break;
+								}
+							}
+							if ((section == none) || (section == virtual_host)) {
+								if (host_setting(key, value, current_host)) {
+									break;
+								} else if (user_setting(key, value, current_host, NULL)) {
+									break;
+								}
+							} else if (section == directory) {
+								if (directory_setting(key, value, current_directory)) {
+									break;
+								}
+							} else if (section == binding) {
+								if (binding_setting(key, value, current_binding)) {
+									break;
+								}
+							} else if (section == fcgi_server) {
+								if (fcgi_server_setting(key, value, current_fcgi_server)) {
+									break;
+								}
+#ifdef HAVE_REWRITE
+							} else if (section == url_rewrite) {
+								if (url_rewrite_setting(key, value, current_rewrite)) {
+									break;
+								}
+#endif
+							}
+							retval = counter;
+						} while (false);
+					}
 				} else {
 					retval = counter;
 				}
@@ -1627,7 +1702,7 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check)
 		if (retval != 0) {
 			break;
 		}
-	} // while
+	} /* while */
 
 	fclose(fp);
 	if (including == false) {
@@ -1737,35 +1812,41 @@ int read_user_configfile(char *configfile, t_host *host, t_tempdata **tempdata)
 
 t_host *get_hostrecord(t_host *host, char *hostname, t_binding *binding) {
 	char *dots;
-	int i, len1, len2;
+	int i, len, len_hostname;
 
 	if (hostname != NULL) {
 		if ((dots = strchr(hostname, ':')) != NULL) {
 			*dots = '\0';
 		}
 
-		len1 = strlen(hostname);
+		if ((len_hostname = strlen(hostname)) > 0) {
+			/* Hostname ends with a dot */
+			if (hostname[len_hostname - 1] == '.') {
+				len_hostname--;
+			}
+		}
+
 		while (host != NULL) {
 			for (i = 0; i < host->hostname.size; i++) {
 				if (host->required_binding.size != 0) {
 					if (in_charlist(binding->binding_id, &(host->required_binding)) == false) {
-						// Binding not allowed
+						/* Binding not allowed */
 						continue;
 					}
 				}
-				if (strcmp(hostname, *(host->hostname.item + i)) == 0) {
-					// Exact match
+				if (strncmp(hostname, *(host->hostname.item + i), len_hostname) == 0) {
+					/* Exact match */
 					return host;
 				} else if (strncmp(*(host->hostname.item + i), "*.", 2) == 0) {
-					// Wildcard in configuration
-					if (strcmp(hostname, *(host->hostname.item + i) + 2) == 0) {
-						// Only domainname requested
+					/* Wildcard in configuration */
+					if (strncmp(hostname, *(host->hostname.item + i) + 2, len_hostname) == 0) {
+						/* Only domainname requested */
 						return host;
 					} else {
-						len2 = strlen(*(host->hostname.item + i));
-						if (len1 >= len2) {
-							if (strcmp(hostname + len1 - len2 + 1, *(host->hostname.item + i) + 1) == 0) {
-								// Wildcard match for hostname
+						len = strlen(*(host->hostname.item + i));
+						if (len_hostname >= len) {
+							if (strncmp(hostname + len_hostname - len + 1, *(host->hostname.item + i) + 1, len - 1) == 0) {
+								/* Wildcard match for hostname */
 								return host;
 							}
 						}
