@@ -40,7 +40,7 @@ typedef struct type_connect_to {
 typedef struct type_cgi_session {
 	t_ipv4        client_ip;
 	t_connect_to  *connect_to;
-	int timer;
+	int           timer;
 
 	struct type_cgi_session *next;
 } t_cgi_session;
@@ -140,6 +140,7 @@ typedef struct type_host {
 	char          *access_logfile;
 	FILE          *access_fileptr;
 	FILE          **access_fp;
+	time_t        access_time;
 	char          *error_logfile;
 	t_charlist    hostname;
 	bool          user_websites;
@@ -263,6 +264,7 @@ int read_main_configfile(char *configfile, t_config *config, bool config_check);
 int read_user_configfile(char *configfile, t_host *host, t_tempdata **tempdata);
 t_host *get_hostrecord(t_host *host, char *hostname, t_binding *binding);
 unsigned short get_throttlespeed(char *type, t_throttle *throttle);
+void close_bindings(t_binding *binding);
 
 #ifdef HAVE_SSL
 #ifndef HAVE_DEV_URANDOM
