@@ -17,6 +17,9 @@
 #include <sys/file.h>
 #include <sys/time.h>
 #include <sys/stat.h>
+#ifdef OPENBSD
+#include <sys/wait.h>
+#endif
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
@@ -41,7 +44,7 @@
 #define MAX_CGI_HEADER          8192
 #define MAX_TRACE_HEADER        1024
 #define VALUE_SIZE                64
-
+#define WAIT_FOR_LOCK              3
 #define FILESIZE_BUFFER_SIZE      30
 #define CGI_BUFFER_SIZE        32768
 
@@ -136,7 +139,7 @@ int send_file(t_session *session) {
 
 			if (invalid_referer) {
 				free(session->file_on_disk);
-				if ((session->file_on_disk = (char*)malloc(strlen(session->host->imgref_replacement) + 4)) != NULL) { // + 3 for ".gz" (gzip encoding)
+				if ((session->file_on_disk = (char*)malloc(strlen(session->host->imgref_replacement) + 4)) != NULL) { /* + 3 for ".gz" (gzip encoding) */
 					strcpy(session->file_on_disk, session->host->imgref_replacement);
 					session->mimetype = get_mimetype(session->file_on_disk, session->config->mimetype);
 				} else {
@@ -228,17 +231,17 @@ int send_file(t_session *session) {
 						if ((send_begin = str2int(range_begin)) >= 0) {
 							if (*range_end != '\0') {
 								if ((send_end = str2int(range_end)) >= 0) {
-									// bytes=XX-XX
+									/* bytes=XX-XX */
 									session->return_code = 206;
 								}
 							} else {
-								// bytes=XX-
+								/* bytes=XX- */
 								session->return_code = 206;
 							}
 						}
 					} else {
 						if ((send_begin = str2int(range_end)) >= 0) {
-							// bytes=-XX
+							/* bytes=-XX */
 							send_begin = file_size - send_begin;
 							session->return_code = 206;
 						}
@@ -419,7 +422,7 @@ int send_file(t_session *session) {
  */
 int execute_cgi(t_session *session) {
 	int retval = 200, result, handle;
-	char *line, *new_line, *contyp_start, *contyp_end, *code;
+	char *line, *new_line, *contyp_start, *contyp_end, *code, c;
 	bool in_body = false, send_in_chunks = true, wrap_cgi;
 #ifdef CYGWIN
 	char *old_path, *win32_path;
@@ -743,10 +746,11 @@ int execute_cgi(t_session *session) {
 
 								retval = -1;
 								while (*line != '\0') {
-									if (*line == '\r') {
+									if ((*line == '\r') || (*line == ' ')) {
+										c = *line;
 										*line = '\0';
 										retval = str2int(code);
-										*line = '\r';
+										*line = c;
 										break;
 									}
 									line++;
@@ -806,9 +810,11 @@ int execute_cgi(t_session *session) {
 											break;
 										}
 									}
+/*
 								} else {
-									//retval = rs_QUIT;
-									//break;
+									retval = rs_QUIT;
+									break;
+*/
 								}
 							} else {
 								if (send_buffer(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
@@ -845,7 +851,7 @@ int execute_cgi(t_session *session) {
 					}
 					retval = 500;
 				}
-		} // switch
+		} /* switch */
 	} while (retval == 200);
 
 	if (session->cgi_type == fastcgi) {
@@ -856,6 +862,12 @@ int execute_cgi(t_session *session) {
 		close(cgi_info.cgi_error);
 	}
 
+#ifdef OPENBSD
+	if (cgi_pid != -1) {
+		waitpid(cgi_pid, NULL, WNOHANG);
+	}
+#endif
+
 	switch (retval) {
 		case rs_DISCONNECT:
 		case rs_FORCE_QUIT:
@@ -1218,7 +1230,7 @@ int handle_trace_request(t_session *session) {
 	buffer[MAX_TRACE_HEADER] = '\0';
 
 	do {
-		// Header
+		/* Header */
 		if (snprintf(buffer, MAX_TRACE_HEADER, "%d\r\nContent-Type: message/http\r\n\r\n", body_size) < 0) {
 			break;
 		} else if (send_header(session, 200) == -1) {
@@ -1229,7 +1241,7 @@ int handle_trace_request(t_session *session) {
 			break;
 		}
 
-		// Body
+		/* Body */
 		if ((code = snprintf(buffer, MAX_TRACE_HEADER, "%s %s", session->method, session->uri)) < 0) {
 			break;
 		} else if (code >= MAX_TRACE_HEADER) {
@@ -1290,7 +1302,7 @@ static t_access allow_alter(t_session *session) {
 }
 
 int handle_put_request(t_session *session) {
-	int handle_write, handle_read = -1, result = -1, total_written = 0;
+	int handle_write, handle_read = -1, result = -1, total_written = 0, lock_timeout;
 	off_t write_begin, write_end, total_size, file_size;
 	ssize_t bytes_read;
 	char *range, *value, *rest, *buffer;
@@ -1325,7 +1337,7 @@ int handle_put_request(t_session *session) {
 	range_found = (range != NULL);
 
 	if ((handle_write = open(session->file_on_disk, O_WRONLY)) == -1) {
-		// New file
+		/* New file */
 		if (range_found) {
 			return 416;
 		}
@@ -1336,23 +1348,32 @@ int handle_put_request(t_session *session) {
 		file_size = NEW_FILE;
 		result = 201;
 	} else {
-		// Existing file
+		/* Existing file */
 		if ((file_size = filesize(session->file_on_disk)) == -1) {
 			close(handle_write);
 			return 500;
 		}
 		result = 204;
 	}
-	if (flock(handle_write, LOCK_EX) == -1) {
-		log_file_error(session, session->file_on_disk, "can't lock file for writing");
-		close(handle_write);
-		if (file_size == NEW_FILE) {
-			unlink(session->file_on_disk);
+	
+	lock_timeout = WAIT_FOR_LOCK;
+	while (fcntl(handle_write, F_SETLK, F_WRLCK) == -1) {
+		if (errno == EINTR) {
+			continue;
+		} else if ((lock_timeout > 0) && ((errno == EACCES) || (errno == EAGAIN))) {
+			lock_timeout--;
+			sleep(1);
+		} else {
+			log_file_error(session, session->file_on_disk, "can't lock file for writing (PUT)");
+			close(handle_write);
+			if (file_size == NEW_FILE) {
+				unlink(session->file_on_disk);
+			}
+			return 500;
 		}
-		return 500;
 	}
 	if (fchmod(handle_write, session->host->alter_fmode) == -1) {
-		log_file_error(session, session->file_on_disk, "can't change file mode");
+		log_file_error(session, session->file_on_disk, "can't change file mode (PUT)");
 	}
 
 	if (range_found) {
@@ -1393,6 +1414,7 @@ int handle_put_request(t_session *session) {
 
 	if ((result == 201) || (result == 204)) {
 		if ((handle_read = open(session->uploaded_file, O_RDONLY)) == -1) {
+			fcntl(handle_write, F_SETLK, F_UNLCK);
 			close(handle_write);
 			if (file_size == NEW_FILE) {
 				unlink(session->file_on_disk);
@@ -1429,7 +1451,8 @@ int handle_put_request(t_session *session) {
 	if (handle_read != -1) {
 		close(handle_read);
 	}
-	flock(handle_write, LOCK_UN);
+	fcntl(handle_write, F_SETLK, F_UNLCK);
+	fsync(handle_write);
 	close(handle_write);
 	if ((result != 201) && (result != 204) && (file_size == NEW_FILE)) {
 		unlink(session->file_on_disk);
