@@ -4,6 +4,7 @@
 #include <stdbool.h>
 #include <unistd.h>
 #include <string.h>
+#include "alternative.h"
 #include "libstr.h"
 #include "libfs.h"
 #ifdef HAVE_REWRITE
@@ -60,14 +61,12 @@ t_line *search_key(t_line *config, char *key) {
 	t_line *result = NULL;
 
 	while (config != NULL) {
-		if (strcmp(config->key, key) == 0) {
+		if (strcasecmp(config->key, key) == 0) {
 			if (config->value == NULL) {
 				printf("'%s' on line %d in '"CONFIG_DIR"/%s' requires a parameter.\n", config->key, config->linenr, config->file);
 				exit(EXIT_FAILURE);
-			} else {
-				if (add_result(&result, config->key, config->value, config->file, config->linenr) != 0) {
-					return NULL;
-				}
+			} else if (add_result(&result, config->key, config->value, config->file, config->linenr) != 0) {
+				return NULL;
 			}
 		}
 		config = config->next;
@@ -76,15 +75,15 @@ t_line *search_key(t_line *config, char *key) {
 	return result;
 }
 
-bool in_result(char *value, t_line *result) {
+t_line *in_result(char *value, t_line *result) {
 	while (result != NULL) {
 		if (strcmp(result->value, value) == 0) {
-			return true;
+			return result;
 		}
 		result = result->next;
 	}
 
-	return false;
+	return NULL;
 }
 
 void dispose_result(t_line *config, bool free_content) {
@@ -101,6 +100,15 @@ void dispose_result(t_line *config, bool free_content) {
 	}
 }
 
+#ifdef DEBUG
+void print_file(t_line *line) {
+	while (line != NULL) {
+		printf("%3d:%-25s [%s] = [%s]\n", line->linenr, line->file, line->key, line->value);
+		line = line->next;
+	}
+}
+#endif
+
 int read_directory(char *dir, t_line **config, t_line **aliases) {
 	t_filelist *filelist, *file;
 	char *path;
@@ -116,10 +124,10 @@ int read_directory(char *dir, t_line **config, t_line **aliases) {
 			if ((path = make_path(dir, file->name)) != NULL) {
 				if (file->is_dir) {
 					retval = read_directory(path, config, aliases);
+					free(path);
 				} else {
 					retval = read_file(path, config, aliases, false);
 				}
-				free(path);
 
 				if (retval == -1) {
 					break;
@@ -241,6 +249,23 @@ bool is_ip_address(char *str) {
 	return (digits == 4);
 }
 
+int find_duplicates(t_line *config, char *key, char *config_dir) {
+	t_line *haystack, *needles, *needle;
+	int errors = 0;
+
+	haystack = needles = search_key(config, key);
+	while (needles != NULL) {
+		if ((needle = in_result(needles->value, needles->next)) != NULL) {
+			printf("Duplicate %s found on line %d in '%s/%s'.\n", key, needle->linenr, config_dir, needle->file);
+			errors++;
+		}
+		needles = needles->next;
+	}
+	dispose_result(haystack, false);
+
+	return errors;
+}
+
 int check_main_config(char *config_dir) {
 	int errors = 0;
 	t_line *config = NULL, *haystack, *needles, *needle;
@@ -263,6 +288,31 @@ int check_main_config(char *config_dir) {
 		return 1;
 	}
 
+	/* Find duplicate ids
+	 */
+	errors += find_duplicates(config, "BindingId", config_dir);
+	errors += find_duplicates(config, "FastCGIid", config_dir);
+	errors += find_duplicates(config, "RewriteId", config_dir);
+
+	/* Binding Id check
+	 */
+	haystack = search_key(config, "bindingid");
+	needles = needle = search_key(config, "requiredbinding");
+	while (needle != NULL) {
+		rest = strdup(needle->value);
+		while (rest != NULL) {
+			split_string(rest, &item, &rest, ',');
+			if (in_result(item, haystack) == NULL) {
+				printf("RequiredBinding '%s' on line %d in '%s/%s' not found in Binding section.\n", item, needle->linenr, config_dir, needle->file);
+				errors++;
+			}
+		}
+		free(rest);
+		needle = needle->next;
+	}
+	dispose_result(haystack, false);
+	dispose_result(needles, false);
+
 	/* FastCGI Id check
 	 */
 	haystack = search_key(config, "fastcgiid");
@@ -271,7 +321,7 @@ int check_main_config(char *config_dir) {
 		rest = strdup(needle->value);
 		while (rest != NULL) {
 			split_string(rest, &item, &rest, ',');
-			if (in_result(item, haystack) == false) {
+			if (in_result(item, haystack) == NULL) {
 				printf("FastCGIid '%s' on line %d in '%s/%s' not found in FastCGIserver section.\n", needle->value, needle->linenr, config_dir, needle->file);
 				errors++;
 			}
@@ -282,16 +332,16 @@ int check_main_config(char *config_dir) {
 	dispose_result(haystack, false);
 	dispose_result(needles, false);
 
-	/* Binding Id check
+	/* Rewrite Id check
 	 */
-	haystack = search_key(config, "bindingid");
-	needles = needle = search_key(config, "requiredbinding");
+	haystack = search_key(config, "rewriteid");
+	needles = needle = search_key(config, "rewriteurl");
 	while (needle != NULL) {
 		rest = strdup(needle->value);
 		while (rest != NULL) {
 			split_string(rest, &item, &rest, ',');
-			if (in_result(item, haystack) == false) {
-				printf("RequiredBinding '%s' on line %d in '%s/%s' not found in Binding section.\n", item, needle->linenr, config_dir, needle->file);
+			if (in_result(item, haystack) == NULL) {
+				printf("RewriteURL '%s' on line %d in '%s/%s' not found in UrlRewrite section.\n", item, needle->linenr, config_dir, needle->file);
 				errors++;
 			}
 		}
@@ -309,10 +359,10 @@ int check_main_config(char *config_dir) {
 		rest = strdup(needle->value);
 		while (rest != NULL) {
 			split_string(rest, &item, &rest, ',');
-			if (in_result(item, haystack) == false) {
+			if (in_result(item, haystack) == NULL) {
 				add_result(&haystack, needle->key, item, needle->file, needle->linenr);
 			} else {
-				printf("Duplicate extension found (%s) in CGIextension.\n", item);
+				printf("Duplicate extension (%s) found in CGIextension.\n", item);
 				errors++;
 			}
 		}
@@ -322,14 +372,22 @@ int check_main_config(char *config_dir) {
 
 	needles = needle = search_key(config, "cgihandler");
 	while (needle != NULL) {
-		rest = strdup(needle->value);
+#ifdef CYGWIN
+		if ((rest = strstr(needle->value, ":\\")) != NULL) {
+			rest += 2;
+		} else
+#endif
+			rest = strdup(needle->value);
 		split_string(rest, &info, &rest, ':');
+#ifdef CYGWIN
+		info = needle->value;
+#endif
 		while (rest != NULL) {
 			split_string(rest, &item, &rest, ',');
-			if (in_result(item, haystack) == false) {
+			if (in_result(item, haystack) == NULL) {
 				add_result(&haystack, needle->key, item, needle->file, needle->linenr);
 			} else {
-				printf("Duplicate extension found (%s) in CGIhandler %s.\n", item, info);
+				printf("Duplicate extension (%s) found in CGIhandler %s.\n", item, info);
 				errors++;
 			}
 		}
@@ -367,7 +425,13 @@ int check_main_config(char *config_dir) {
 		if ((strcmp(haystack->key, "cgiextension") == 0) || (strcmp(haystack->key, "extension") == 0)) {
 			has_dot = (strchr(haystack->value, '.') != NULL);
 		} else if (strcmp(haystack->key, "cgihandler") == 0) {
-			if ((rest = strchr(haystack->value, ':')) != NULL) {
+#ifdef CYGWIN
+			if ((info = strstr(haystack->value, ":\\")) != NULL) {
+				info += 2;
+			} else
+#endif
+				info = haystack->value;
+			if ((rest = strchr(info, ':')) != NULL) {
 				has_dot = (strchr(rest, '.') != NULL);
 			} else {
 				has_dot = false;
@@ -395,6 +459,7 @@ void check_url_rewrite(char *config_dir, char **rewrite_id) {
 	char input[MAX_INPUT + 1], **id, *url, *new_url;
 	t_url_rewrite *url_rewrite, *rewrite = NULL;
 	bool in_rule_section = false;
+	int result;
 
 	if (chdir(config_dir) == -1) {
 		perror(config_dir);
@@ -461,25 +526,34 @@ void check_url_rewrite(char *config_dir, char **rewrite_id) {
 	/* Start testing
 	 */
 	input[MAX_INPUT] = '\0';
-	printf("\nType 'exit' to leave the program.\n\nurl: ");
+	printf("\n===[ URL rewrite tester\n");
+	printf("Use empty input to leave the program.\n\nurl: ");
 	while (fgets(input, MAX_INPUT, stdin) != NULL) {
 		url = remove_spaces(input);
 
-		if ((strcmp(url, "exit") == 0) || (strcmp(url, "quit") == 0)) {
+		if (strcmp(url, "") == 0) {
+			printf("bye!\n\n");
 			break;
 		}
 
 		id = rewrite_id;
 		while (*id != NULL) {
-			rewrite_url(url, *id, url_rewrite, &new_url);
+			if ((result = rewrite_url(url, *id, url_rewrite, &new_url)) == RW_ERROR) {
+				perror("rewrite_url()");
+			}
 			if (new_url != NULL) {
 				url = new_url;
 			}
+			if (result == RW_DENY_ACCESS) {
+				url = "(403 Forbidden)";
+				break;
+			}
 			id++;
 		}
 
 		if (new_url != NULL) {
 			printf("new: %s\n\n", new_url);
+			free(new_url);
 		} else {
 			printf("old: %s\n\n", url);
 		}
@@ -495,7 +569,7 @@ void show_help(char *wigwam) {
 	printf("         -h: show this information and exit.\n");
 	printf("         -q: don't print the test results.\n");
 #ifdef HAVE_REWRITE
-	printf("         -r [<rewrite_id> ...]: test URL rewriting rule(s).\n");
+	printf("         -r [<rewrite_id> ...]: test URL rewrite rule(s).\n");
 #endif
 	printf("         -v: show version and exit.\n");
 }
